/* This program generates a PWM of varying duty-cycles on OC2*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>


/* NOTE: instead of using "unsigned char" you can include <inttypes.h> and use "uint8_t"
 * other examples: unsigned int -> uint16_t or int -> int16_t
 */

volatile uint8_t width; /* positive pusle width */
volatile uint8_t delay; /* used to slow the pulse width changing */

SIGNAL (SIG_OVERFLOW2) 
{
	if(delay++ == 20)
	{
		OCR2 = width++;
		delay = 0;
	}
}


int main (void)
{	
	/* must make OC2 pin an output for the PWM to visible */
	DDRD = _BV(DDD7);

	/* use Timer 2 FastPWM and the overflow interrupt to update duty-cycle */
    TCCR2 = _BV (WGM21) | _BV (WGM20) | _BV (COM21) | _BV(COM20) | _BV(CS21) | _BV(CS20);
	TIMSK = _BV (TOIE2);
 
	/* setup initial conditions */
	delay = 0;
	
    /* enable interrupts */
    sei ();

    for (;;)
	{
        ; /* LOOP FOREVER as the interrupt will make necessary adjustment */
	}

    return (0);
}




